@@ -1,7 +1,7 @@
 /*
     webalizer - a web server log analysis program
 
-    Copyright (C) 1997-2000  Bradford L. Barrett (brad@mrunix.net)
+    Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -101,9 +101,9 @@ char    *our_gzgets(gzFile, char *, int);           /* our gzgets          */
 /*********************************************/
 
 char    *version     = "2.01";                /* program version          */
-char    *editlvl     = "06";                  /* edit level               */
-char    *moddate     = "17-Oct-2000";         /* modification date        */
-char    *copyright   = "Copyright 1997-2000 by Bradford L. Barrett";
+char    *editlvl     = "09";                  /* edit level               */
+char    *moddate     = "23-Oct-2001";         /* modification date        */
+char    *copyright   = "Copyright 1997-2001 by Bradford L. Barrett";
 
 int     verbose      = 2;                     /* 2=verbose,1=err, 0=none  */ 
 int     debug_mode   = 0;                     /* debug mode flag          */
@@ -724,18 +724,23 @@ int main(int argc, char *argv[])
             lptr=lptr->next;
          }
 
+         /* unescape referrer */
+         unescape(log_rec.refer);
+
          /* fix referrer field */
          cp1 = log_rec.refer;
          cp3 = cp2 = cp1++;
          if ( (*cp2 != '\0') && (*cp2 == '"') )
          {
-            while ( *cp1 != '\0' ) { cp3 = cp2; *cp2++ = *cp1++; }
+            while ( *cp1 != '\0' )
+            {
+               cp3=cp2;
+               if (*cp1<32 || *cp1>=127 || *cp1=='<') *cp1=0;
+               else *cp2++=*cp1++;
+            }
             *cp3 = '\0';
          }
 
-         /* unescape referrer */
-         unescape(log_rec.refer);
-
          /* strip query portion of cgi referrals */
          cp1 = log_rec.refer;
          if (*cp1 != '\0')
@@ -817,10 +822,10 @@ int main(int argc, char *argv[])
 		    /* not for now */
 		}
 	    } else {
-		cp1=strstr(str,"Opera");  /* Netscape flavor      */
+		cp1=strstr(str,"Opera");  /* Opera flavor         */
 		if (cp1!=NULL)
 		{
-		    while (*cp1!=' '&&*cp1!=' '&&*cp1!='\0') *cp2++=*cp1++;
+		    while (*cp1!='/'&&*cp1!=' '&&*cp1!='\0') *cp2++=*cp1++;
 		    while (*cp1!='.'&&*cp1!='\0') *cp2++=*cp1++;
 		    if (mangle_agent<5)
 		    {
@@ -995,8 +1000,9 @@ int main(int argc, char *argv[])
          cp1 = log_rec.hostname;
          while (*cp1 != '\0')
          {
-            if ( (*cp1>='A') && (*cp1<='Z')) *cp1 += 'a'-'A';
-            cp1++;
+            if ( (*cp1>='A') && (*cp1<='Z') ) *cp1 += 'a'-'A';
+            if ( (isalnum(*cp1))||(*cp1=='.')||(*cp1=='-') ) cp1++;
+            else *cp1='\0';
          }
 
          /* Ignore/Include check */
@@ -1401,7 +1407,7 @@ void get_config(char *fname)
                      "IncludeReferrer",   /* Referrers to include       47  */
                      "IncludeAgent",      /* User Agents to include     48  */
                      "PageType",          /* Page Type (pageview)       49  */
-                     "VisitTimeout",      /* Visit timeout (HHMMSS)     50  */
+                     "VisitTimeout",      /* Visit timeout (seconds)    50  */
                      "GraphLegend",       /* Graph Legends (yes/no)     51  */
                      "GraphLines",        /* Graph Lines (0=none)       52  */
                      "FoldSeqErr",        /* Fold sequence errors       53  */
